<?php

namespace Drupal\view_entity_render_popup\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for rendering entities in a modal popup.
 */
class EntityRenderPopupController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new EntityRenderPopupController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Renders the entity in a modal popup.
   */
  public function renderEntity(Request $request, $entity_type, $entity_id, $view_mode) {
    $response = new AjaxResponse();
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($entity_id);

    if ($entity instanceof EntityInterface) {
      $view_builder = $this->entityTypeManager->getViewBuilder($entity_type);
      $build = $view_builder->view($entity, $view_mode);
      $options = [
        'classes' => [
          'ui-dialog' => 'views-ui-dialog js-views-ui-dialog',
          'view-entity-render-popup' => 'view-entity-render-popup',
        ],
        'width' => '80%',
      ];
      $response->addCommand(new OpenModalDialogCommand($entity->label(), $build, $options));
    }

    return $response;
  }

}