<?php

namespace Drupal\Tests\view_entity_render_popup\Functional;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the View Entity Render Popup field formatter.
 *
 * @group view_entity_render_popup
 */
class ViewEntityRenderPopupFormatterTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = ['node', 'field', 'view_entity_render_popup'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a content type.
    NodeType::create(['type' => 'article', 'name' => 'Article'])->save();

    // Create an entity reference field.
    FieldStorageConfig::create([
      'field_name' => 'field_ref_node',
      'entity_type' => 'node',
      'type' => 'entity_reference',
      'settings' => ['target_type' => 'node'],
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_ref_node',
      'entity_type' => 'node',
      'bundle' => 'article',
      'label' => 'Reference Node',
      'settings' => ['handler' => 'default:node'],
    ])->save();

    // Configure the field to use the custom formatter.
    $form_display = EntityFormDisplay::create([
      'targetEntityType' => 'node',
      'bundle' => 'article',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $form_display->setComponent('field_ref_node', [
      'type' => 'view_entity_render_popup',
      'settings' => [
        'view_mode' => 'teaser',
        'link_text' => 'View Node',
      ],
    ])->save();
  }

  /**
   * Tests the field formatter rendering and modal functionality.
   */
  public function testViewEntityRenderPopupFormatter() {
    // Create a referenced node.
    $referenced_node = Node::create([
      'title' => 'Referenced Node',
      'type' => 'article',
      'status' => 1,
    ]);
    $referenced_node->save();

    // Create a node with the entity reference field.
    $node = Node::create([
      'title' => 'Test Node',
      'type' => 'article',
      'status' => 1,
      'field_ref_node' => ['target_id' => $referenced_node->id()],
    ]);
    $node->save();

    // Log in as an admin user.
    $user = $this->drupalCreateUser(['access content']);
    $this->drupalLogin($user);

    // Access the node page.
    $this->drupalGet('node/' . $node->id());
    $this->assertSession()->statusCodeEquals(200);

    // Verify the link text and modal attributes.
    $this->assertSession()->linkExists('View Node');
    $this->assertSession()->elementExists('css', 'a.use-ajax[data-dialog-type="modal"]');

    // Simulate clicking the link to open the modal (basic check for URL).
    $link = $this->assertSession()->elementExists('css', 'a.use-ajax');
    $href = $link->getAttribute('href');
    $this->assertStringContainsString('/view-entity-render-popup/node/' . $referenced_node->id() . '/teaser', $href);
  }

}